@@ -1,10 +1,10 @@
 ;;; ox-linuxmag-fr.el --- Org-mode exporter for the French GNU/Linux Magazine -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2022 Damien Cassou
+;; Copyright (C) 2022-2023 Damien Cassou
 
 ;; Authors: Damien Cassou <damien@cassou.me>
-;; Package-Version: 0.3.0
-;; Package-Revision: 0.3.0-0-g2c06d5441e9e
+;; Package-Version: 20250907.1103
+;; Package-Revision: 67381be882a3
 ;; URL: https://github.com/DamienCassou/ox-linuxmag-fr
 ;; Package-Requires: ((emacs "28.1"))
 ;; Created: 15 Oct 2022
@@ -59,11 +59,13 @@
   `((template . ox-linuxmag-fr--template)
     (bold . ox-linuxmag-fr--bold)
     (code . ox-linuxmag-fr--code)
+    (verbatim . ox-linuxmag-fr--code)
     (headline . ox-linuxmag-fr--headline)
     (italic . ox-linuxmag-fr--italic)
     (item . ox-linuxmag-fr--format-contents)
     (link . ox-linuxmag-fr--link)
     (paragraph . ox-linuxmag-fr--paragraph)
+    (plain-text . ox-linuxmag-fr--plain-text)
     (plain-list . ox-linuxmag-fr--format-contents)
     (special-block . ox-linuxmag-fr--special-block)
     (src-block . ox-linuxmag-fr--src-block)
@@ -71,8 +73,8 @@
     (table-cell . ox-linuxmag-fr--table-cell)
     (target . ox-linuxmag-fr--target)
     (underline . ox-linuxmag-fr--underline))
-  :menu-entry '(?g "Export to ODT for GNU/Linux Magazine"
-                   ((?g "As ODT file" ox-linuxmag-fr-export-to-odt)))
+  :menu-entry '(?G "Export to ODT for GNU/Linux Magazine"
+                   ((?G "As ODT file" ox-linuxmag-fr-export-to-odt)))
   :options-alist
   `((:author-description "AUTHOR_DESCRIPTION" nil nil newline)
     (:logos "LOGOS" nil nil newline))
@@ -280,7 +282,7 @@ CONTENTS is the text with italic markup.  INFO is a plist holding
 contextual information."
   (ox-linuxmag-fr--format-textspan contents "italic"))
 
-(defun ox-linuxmag-fr--link (link _desc info)
+(defun ox-linuxmag-fr--link (link desc info)
   "Transcode a LINK object from Org to ODT.
 
 DESC is the description part of the link, or the empty string.
@@ -288,7 +290,7 @@ INFO is a plist holding contextual information.  See
 `org-export-data'."
   (let ((raw-link (org-element-property :raw-link link)))
     (cl-case (intern (org-element-property :type link))
-      (fuzzy (ox-linuxmag-fr--format-fuzzy-link link info))
+      (fuzzy (ox-linuxmag-fr--format-fuzzy-link link desc info))
       ;; file links are taken care of by the containing paragraph:
       (file nil)
       (t (ox-linuxmag-fr--format-textspan raw-link "url")))))
@@ -308,57 +310,85 @@ INFO is a plist holding contextual information."
              (ox-linuxmag-fr--is-paragraph-a-figure-p element info))
     (org-export-get-ordinal element info nil #'ox-linuxmag-fr--is-paragraph-a-figure-p)))
 
-(defun ox-linuxmag-fr--format-fuzzy-link (link info)
+(defun ox-linuxmag-fr--format-fuzzy-link (link desc info)
   "Return a string representing LINK.
 
 If LINK targets a figure, the returned string is the one-based
 position of the figure within the document.
 
+DESC is the description part of the link, or the empty string.
+
 INFO is a plist holding contextual information."
   (let* ((target (org-export-resolve-fuzzy-link link info)))
     (if (and (eq (org-element-type target) 'paragraph)
              (ox-linuxmag-fr--is-paragraph-a-figure-p target info))
         (format (number-to-string (ox-linuxmag-fr--figure-number target info)))
-      (ox-linuxmag-fr--format-textspan (format "[%s]" (org-element-property :raw-link link)) "gras"))))
+      (let* ((ref-string (ox-linuxmag-fr--format-textspan
+                          (format "[%s]" (org-element-property :raw-link link)) "gras")))
+        (if (length= desc 0)
+            ref-string
+          (concat desc " " ref-string))))))
 
 (defun ox-linuxmag-fr--paragraph (paragraph contents info)
   "Transcode a PARAGRAPH element from Org to ODT.
 CONTENTS is the contents of the paragraph, as a string.  INFO is
 the plist used as a communication channel."
-  (let ((note-type (org-export-read-attribute :attr_linuxmag-fr paragraph :note)))
-    (cond
-     (note-type
-      (ox-linuxmag-fr--format-note contents note-type))
-     ((ox-linuxmag-fr--is-paragraph-a-figure-p paragraph info)
-      (ox-linuxmag-fr--format-figure paragraph contents info))
-     (t
-      (let* ((parent (org-export-get-parent paragraph))
-             (parent-type (org-element-type parent))
-             (prefix (when (eq parent-type 'item) "- ")))
-        (org-odt--format-paragraph
-         paragraph
-         (concat prefix contents)
-         info
-         "Normal" "" ""))))))
+  (cond
+   ((ox-linuxmag-fr--is-paragraph-a-figure-p paragraph info)
+    (ox-linuxmag-fr--format-figure paragraph contents info))
+   (t
+    (let* ((parent (org-export-get-parent paragraph))
+           (parent-type (org-element-type parent))
+           (prefix (when (eq parent-type 'item)
+                     (ox-linuxmag-fr--make-item-bullet (org-export-get-parent paragraph))))
+           (style (if (and
+                       (eq parent-type 'special-block)
+                       (equal (org-element-property :type parent) "note_pao"))
+                      "pragma"
+                    "Normal")))
+      (org-odt--format-paragraph
+       paragraph
+       (concat prefix contents)
+       info
+       style
+       "" "")))))
+
+(defun ox-linuxmag-fr--make-item-bullet (item)
+  "Return the bullet appropriate for list-item ITEM."
+  (let* ((plain-list (org-export-get-parent item))
+	 (list-type (org-element-property :type plain-list)))
+    (pcase list-type
+      (`ordered
+       (let* ((struct (org-element-property :structure item))
+	      (bul (org-list-bullet-string (org-element-property :bullet item)))
+	      (num (number-to-string
+		    (car (last (org-list-get-item-number
+				(org-element-property :begin item)
+				struct
+				(org-list-prevs-alist struct)
+				(org-list-parents-alist struct)))))))
+	 (replace-regexp-in-string "[0-9A-Za-z]+" num bul)))
+      (`unordered (org-list-bullet-string (org-element-property :bullet item)))
+      (_ (user-error "Plain-list type '%s' isn't accepted" list-type)))))
 
 (defun ox-linuxmag-fr--format-note (contents note-type)
   "Return a string containing CONTENTS with a box markup.
 
-NOTE-TYPE is a string representing the kind of box."
-  (cl-case (intern note-type)
-    ('PAO (concat
-           (ox-linuxmag-fr--format-pragma "Début note PAO")
-           (ox-linuxmag-fr--format-textp contents "pragma")
-           (ox-linuxmag-fr--format-pragma "Fin note PAO")))
+NOTE-TYPE is a symbol representing the kind of box."
+  (cl-case note-type
+    (PAO (concat
+          (ox-linuxmag-fr--format-pragma "Début note PAO")
+          contents
+          (ox-linuxmag-fr--format-pragma "Fin note PAO")))
     ((attention avertissement)
      (concat
-      (ox-linuxmag-fr--format-pragma (format "Début note : %s" (capitalize note-type)))
-      (ox-linuxmag-fr--format-textp contents)
+      (ox-linuxmag-fr--format-pragma (format "Début note : %s" (capitalize (symbol-name note-type))))
+      contents
       (ox-linuxmag-fr--format-pragma "Fin note")))
     (t
      (concat
       (ox-linuxmag-fr--format-pragma (format "Début note"))
-      (ox-linuxmag-fr--format-textp contents)
+      contents
       (ox-linuxmag-fr--format-pragma "Fin note")))))
 
 (defun ox-linuxmag-fr--format-figure (paragraph _contents info)
@@ -391,6 +421,10 @@ contextual information."
   (let ((type (intern (org-element-property :type special-block))))
     (cl-case type
       (encadre (ox-linuxmag-fr--format-encadre special-block contents))
+      (note (ox-linuxmag-fr--format-note contents 'default))
+      (note_pao (ox-linuxmag-fr--format-note contents 'PAO))
+      (note_avertissement (ox-linuxmag-fr--format-note contents 'avertissement))
+      (note_attention (ox-linuxmag-fr--format-note contents 'attention))
       (t contents))))
 
 (defun ox-linuxmag-fr--format-encadre (special-block contents)
@@ -410,33 +444,50 @@ CONTENTS holds the contents of the item.  INFO is a plist holding
 contextual information."
   (let ((block-type (or (org-export-read-attribute :attr_linuxmag-fr src-block :type)
                         "code"))
-        (prompt (or (org-export-read-attribute :attr_linuxmag-fr src-block :prompt)
-                    "")))
-    (if (and (length> prompt 2) (equal (elt prompt 0) ?'))
-        (setq prompt (string-trim prompt "'" "'")))
+        (info (ox-linuxmag-fr--src-block-rewrite-info info)))
     (mapconcat
-     (lambda (line)
-       (ox-linuxmag-fr--format-textp
-        (let* ((ox-linuxmag-fr--inline-code-style "code_5f_em")
-               (text (org-export-data (org-element-parse-secondary-string line '(code) src-block)
-                                      (append '(:with-special-strings nil) info)))
-               (deindented-text (string-trim-left text))
-               (indentation-length (- (length text) (length deindented-text))))
-          (format "%s%s%s"
-                  prompt
-                  (if (> indentation-length 0) (format "<text:s text:c=\"%s\"/>" indentation-length) "")
-                  deindented-text))
-        block-type))
+     (lambda (line) (ox-linuxmag-fr--src-block-convert-line line src-block block-type info))
      (org-split-string (org-element-property :value src-block) "\n")
      "\n")))
 
+(defun ox-linuxmag-fr--src-block-rewrite-info (info)
+  "Rewrite INFO so spaces in plain-text strings are preserved.
+
+INFO is a plist holding contextual information."
+  (let* ((identity-transcoder (lambda (data _info) (org-odt--encode-plain-text data)))
+         (initial-translate-alist (plist-get info :translate-alist))
+         (translate-alist `((plain-text . ,identity-transcoder)
+                            ,@initial-translate-alist)))
+    `(:translate-alist ,translate-alist ,@info)))
+
+(defun ox-linuxmag-fr--src-block-convert-line (line src-block block-type info)
+  "Convert string LINE into ODT.
+
+SRC-BLOCK is the Org parsed element containing this
+line.  BLOCK-TYPE is either \"console\" or \"code\".  INFO is a
+plist holding contextual information."
+  (ox-linuxmag-fr--format-textp
+   (let* ((ox-linuxmag-fr--inline-code-style "code_5f_em"))
+     (org-export-data
+      (org-element-parse-secondary-string line '(code) src-block)
+      info))
+   block-type))
+
+(defun ox-linuxmag-fr--plain-text (text info)
+  "Transcode a TEXT string from Org to ODT.
+TEXT is the string to transcode.  INFO is a plist holding
+contextual information."
+  ;; deactivate automatic character transformations:
+  (let ((org-odt-special-string-regexps '()))
+    (org-odt-plain-text text info)))
+
 (defun ox-linuxmag-fr--table (table contents info)
   "Transcode a TABLE element from Org to ODT.
 CONTENTS holds the contents of the item.  INFO is a plist holding
 contextual information."
   (concat
-   (format "<table:table table:style-name=\"Tableau\">")
-   (format "<table:table-column table:style-name=\"Tableau.A\" table:number-columns-repeated=\"%s\"/>"
+   (format "<table:table table:style-name=\"Tableau1\">")
+   (format "<table:table-column table:style-name=\"Tableau1.A\" table:number-columns-repeated=\"%s\"/>"
            (length (org-odt-table-first-row-data-cells table info)))
    contents
    (format "</table:table>")))
@@ -446,8 +497,8 @@ contextual information."
 CONTENTS is nil.  INFO is a plist used as a communication
 channel."
   (concat
-   (format "<table:table-cell table:style-name=\"Tableau.A1\" office:value-type=\"string\">")
-   (ox-linuxmag-fr--format-textp contents)
+   (format "<table:table-cell table:style-name=\"Tableau1.A1\" office:value-type=\"string\">")
+   (when contents (ox-linuxmag-fr--format-textp contents))
    (format "</table:table-cell>")))
 
 (defun ox-linuxmag-fr--target (target _contents _info)
@@ -587,7 +638,7 @@ added to the archive."
 
 (defun ox-linuxmag-fr--add-files-to-archive-directory (archive-directory odt-file figure-files)
   "Add ODT-FILE and FIGURE-FILES to ARCHIVE-DIRECTORY."
-  (rename-file odt-file archive-directory)
+  (copy-file odt-file archive-directory)
   (ox-linuxmag-fr--copy-figure-files-to-archive-directory figure-files archive-directory))
 
 (defun ox-linuxmag-fr--copy-figure-files-to-archive-directory (figure-files archive-directory)
